/******************************************************************************
*
* project name:    TI-Puzzlize
* file name:       board.c
* initial date:    16/11/2000
* authors:         thomas.nussbaumer@gmx.net
*                  marcos.lopez@wol.es
*
* description:     contains board handling and drawing functions
*
* $Id: board.c,v 1.3 2004/06/30 12:37:21 DEBROUX Lionel Exp $
*
******************************************************************************/

#define SHUFFLE_ROUNDS 100   // number of random exchangements

void PrintTime();


// offsets (mainly used for TI92plus)
short offsetx = 0;
short offsety = 0;

// the board and the back buffers
short*         board         = NULL;
unsigned char* backbuffer[2] = {NULL,NULL};


// how much moves done
unsigned long moves_done = 0;

short cursor_x  = 0;
short cursor_y  = 0;


/*---------------------------------------------------------------------------*/
/* here comes the sprite data and the tiles generation routines              */
/*---------------------------------------------------------------------------*/
#include "gfxdata.c"

unsigned long mask[32] = {
    0xc0000000,0x80000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000000,0x00000000,
    0x00000000,0x00000000,0x00000001,0x00000003
};


/*===========================================================================*/
/* allocate space for some necessary globals                                 */
/*===========================================================================*/
short AllocateVariables() {
    if (!(board = malloc(BOARD_FIELDS*sizeof(short)))) return 0;
    if (!(backbuffer[0] = malloc(LCD_SIZE)))         return 0;
    if (!(backbuffer[1] = malloc(LCD_SIZE)))         return 0;

    if (!(AllocateTiles())) return 0;

    if (TI89) offsetx = offsety = 0;
    else      offsetx = (240-160)/2,offsety = (128-100)/2;

    return 1;
}


/*===========================================================================*/
/* free allcated global again (CALL ONLY ONCE !!!)                           */
/*===========================================================================*/
void FreeVariables() {
    if (board)         free(board);
    if (backbuffer[0]) free(backbuffer[0]);
    if (backbuffer[1]) free(backbuffer[1]);
    CleanupTiles();
}



/*===========================================================================*/
/* initialize the board                                                      */
/*===========================================================================*/
void InitBoard() {
    short  i = 0;
    short *tmpboard  = board;
    unsigned long buffer[960];

    //------------------------------
    // generate the tiles
    //------------------------------
    if (GetPicture((unsigned char*)buffer)) {
        MakeTiles(buffer);
    }
    else {
        MakeTiles(NULL);
    }

    //------------------------------
    // fill up the board
    //------------------------------
    for (i=0;i<BOARD_FIELDS-1;i++) *tmpboard++ = i % BOARD_FIELDS;

    *tmpboard = -1; // last field is empty

    cursor_x  = BOARD_SIZE_X-1;  // cursor is on the last field
    cursor_y  = BOARD_SIZE_Y-1;  // cursor is on the last field
}


/*===========================================================================*/
/* draws a single field                                                      */
/*===========================================================================*/
void DrawField(short x,short y) {
    short i;

    short which = board[y*BOARD_SIZE_X+x];

    if (which != -1) {
        short offset = which * 32;
        short xx = offsetx+x*(13+16)+11-5;
        short yy = offsety+y*(13+16)+10-5-2;
        for (i=0;i<2;i++) {
            Sprite32(xx,yy,32,mask,           backbuffer[i],SPRT_AND);
            Sprite32(xx,yy,32,tiles[i]+offset,backbuffer[i],SPRT_OR);
        }
    }
}


/*===========================================================================*/
/* generates board display in backbuffer and copies it to display afterwards */
/*===========================================================================*/
void DrawBoard(short outtime) {
    short x,y;

    for (x=0;x<2;x++) memset(backbuffer[x],0,LCD_SIZE);

    for (y=0;y<BOARD_SIZE_Y;y++) {
        for (x=BOARD_SIZE_X-1;x>=0;x--) {
            DrawField(x,y);
        }
    }

    for (x=0;x<2;x++) memcpy(GetPlane(x),backbuffer[x],LCD_SIZE);
    if (outtime) PrintTime();
}


/*===========================================================================*/
/* useful macros to get and set stones                                       */
/*===========================================================================*/
#define GetStone(x,y) board[BOARD_SIZE_X*(y)+(x)]
#define SetStone(x,y,t) board[BOARD_SIZE_X*(y)+(x)] = (t)


/*===========================================================================*/
/* check if game is won (if every pieces are in their correct positions)     */
/*===========================================================================*/
short CheckWin() {
    short i;
    short *b = board;

    for (i=0;i<BOARD_FIELDS-1;i++) {
        if (*b++ != i) return 0;
    }

    return 1;
}


/*===========================================================================*/
/* shuffle the board                                                         */
/*===========================================================================*/
void ShuffleBoard() {
    short i;
    short dir;
    short valid;
    short stone;
    short axis = 0;

    for (i=0;i<SHUFFLE_ROUNDS;i++) {
        valid = 0;
        //---------------------------------------------------
        // make a move into a random, but valid direction ...
        //---------------------------------------------------
        while (!valid) {
            dir = random(2);

            if (axis) dir += 2;
            switch(dir) {
                case 0:
                    if (!cursor_x) continue;
                    cursor_x--;
                    stone = GetStone(cursor_x,cursor_y);
                    SetStone(cursor_x+1,cursor_y,stone);
                    SetStone(cursor_x,cursor_y,-1);
                    valid = 1;
                    axis = !axis;
                    break;
                case 1:
                    if (cursor_x==BOARD_SIZE_X-1) continue;
                    cursor_x++;
                    stone = GetStone(cursor_x,cursor_y);
                    SetStone(cursor_x-1,cursor_y,stone);
                    SetStone(cursor_x,cursor_y,-1);
                    valid = 1;
                    axis = !axis;
                    break;
                case 2:
                    if (!cursor_y) continue;
                    cursor_y--;
                    stone = GetStone(cursor_x,cursor_y);
                    SetStone(cursor_x,cursor_y+1,stone);
                    SetStone(cursor_x,cursor_y,-1);
                    valid = 1;
                    axis = !axis;
                    break;
                case 3:
                    if (cursor_y==BOARD_SIZE_Y-1) continue;
                    cursor_y++;
                    stone = GetStone(cursor_x,cursor_y);
                    SetStone(cursor_x,cursor_y-1,stone);
                    SetStone(cursor_x,cursor_y,-1);
                    valid = 1;
                    axis = !axis;
                    break;
            }
        }
        DrawBoard(0);
    }
}


//=============================================================================
// Revision History
//=============================================================================
//
// $Log: board.c,v $
// Revision 1.3  2004/06/30 12:37:21  DEBROUX Lionel
// changes for TI-89T/TIGCC 0.95
//
// Revision 1.2  2002/11/14 14:36:56  nussbaumer
// changes for AMS2.08/V200/TIGCC 0.94
//
// Revision 1.1  2000/12/19 10:56:02  Thomas Nussbaumer
// initial version
//
//
